import React, { useState, useEffect } from 'react';
import { useDispatch, useSelector } from 'react-redux';
import axios from "axios";
import { Formik, Form, Field, ErrorMessage } from 'formik';
import * as Yup from 'yup';
import { useRouter } from 'next/router'
import { ConnectedFocusError } from 'focus-formik-error';
import { Elements, ElementsConsumer, useStripe } from "@stripe/react-stripe-js";
import { loadStripe } from "@stripe/stripe-js/pure";
import { pendingCart, getCart, updateBilling, updateShipping, updateAddressStatus, updateShippingMethod, isCartStatusUpdate } from "../../reducer/woocommerceSlice";
import { FormObserver } from "../../utils/checkout";
import storeApi from "../../helper/StoreApi";
import AddressApi from "../../helper/AddressApi";
import { fetchShippingCountries, fetchPaymentGateways } from '../../services';
import CardForm from '../stripe/cardform';
import CheckoutInstructions from '../popup/CheckoutInstructions';
import VisaIconCheckout from '../../assets/card-visa.svg';
import MasterIconCheckout from '../../assets/card-mastercard.svg';
import AmericanIconCheckout from '../../assets/card-amex.svg';
import 'select2/dist/css/select2.min.css';
import 'select2';
import styles from '../../styles/checkout/checkoutdetails.module.scss';


const CheckoutDetails = () => {
    const router = useRouter()
    //const stripePromise = loadStripe("pk_test_51NSeSxK5WtqRzcU32BA8opiSUKlus7gbWa1Kce8jzf8a3znUOg3weEwHaAHLx2zVw1J2DgsgRSRof2KtzXNjNaWH00S4HSNXCa");
    const stripe = useStripe()
    const { cart, isCartReady, checkout, stripe_payment_data, saveStripeCardToken } = useSelector(state => state.woocommerce);
    const { userIsLogin, userIsLoading, userData } = useSelector(state => state.user);
    const dispatch = useDispatch();


    const [shippingCountriesData, setShippingCountriesData] = useState([]);
    const [paymentGatewaysData, setPaymentGatewaysData] = useState([]);
    const [modalIsOpen, setIsOpen] = useState(false);
    const [userAddressData, setUserAddressData] = useState([]);
    const [couponMessage, setCouponMessage] = useState('');
    const [couponMessageError, setCouponMessageError] = useState('');
    const [couponBtnText, setCouponBtnText] = useState('Submit');
    const [couponCode, setCouponCode] = useState('');
    const [checkoutBtnText, setCheckoutBtnText] = useState('Continue to Payment');
    const [shippingAddressSelected, setShippingAddressSelected] = useState('');
    const [billingAddressSelected, setBillingAddressSelected] = useState('');
    const [errorMessage, setErrorMessage] = useState('');
    const [paymentCompleted, setPaymentCompleted] = useState(false)

    function openModal() {
        setIsOpen(true);
    }

    function closeModal() {
        setIsOpen(false);
    }

    const buildRequiredForDiffAddress = requiredText => ({
        is: true,
        then: () => Yup.string()
            .required(requiredText)
    });

    const CheckoutSchema = Yup.object().shape({
        billing_first_name: Yup.string().required("Billing First Name is a required field."),
        billing_last_name: Yup.string().required("Billing Last Name is a required field."),
        billing_postcode: Yup.string().required("Billing Postcode is a required field."),
        billing_country: Yup.string().required("Billing Country is a required field."),
        billing_address_1: Yup.string().required("Billing Address Line 1 is a required field."),
        billing_city: Yup.string().required("Billing City is a required field."),
        billing_state: Yup.string().required("Billing State is a required field."),
        billing_phone: Yup.string().required("Billing Telephone Number is a required field."),
        billing_email: Yup.string().required("Billing Email Address is a required field."),
        payment_method: Yup.string().required("Payment method is a required field."),
        terms: Yup.boolean()
            .required("Please read and accept the terms and conditions to proceed with your order.")
            .oneOf([true], "Please read and accept the terms and conditions to proceed with your order."),
        ship_to_different_address: Yup.boolean(),
        shipping_first_name: Yup.string().when("ship_to_different_address", buildRequiredForDiffAddress("Shipping First Name is a required field.")),
        shipping_last_name: Yup.string().when("ship_to_different_address", buildRequiredForDiffAddress("Shipping Last Name is a required field.")),
        shipping_postcode: Yup.string().when("ship_to_different_address", buildRequiredForDiffAddress("Shipping Postcode is a required field.")),
        shipping_country: Yup.string().when("ship_to_different_address", buildRequiredForDiffAddress("Shipping Country is a required field.")),
        shipping_address_1: Yup.string().when("ship_to_different_address", buildRequiredForDiffAddress("Shipping Addres Line 1 is a required field.")),
        shipping_city: Yup.string().when("ship_to_different_address", buildRequiredForDiffAddress("Shipping City is a required field.")),
        shipping_state: Yup.string().when("ship_to_different_address", buildRequiredForDiffAddress("Shipping State is a required field.")),
        shipping_phone: Yup.string().when("ship_to_different_address", buildRequiredForDiffAddress("Shipping Telephone Number is a required field.")),
        shipping_email: Yup.string().when("ship_to_different_address", buildRequiredForDiffAddress("Shipping Email Address is a required field.")),
    });

    useEffect(() => {

        if (cart && cart.items && cart.items.length == 0 && isCartReady) {
            window.location.href = '/basket?redirect=checkout';
        }

        if (shippingCountriesData.length == 0) {
            handleShippingCountries();
        }

        if (paymentGatewaysData.length == 0) {
            handlePaymentGateways();
        }



    }, [isCartReady]);

    useEffect(() => {

        if (userIsLoading == false && userIsLogin == true) {
            handleUserAddress(userData.id);
        }

    }, [userIsLoading]);






    const handleShippingCountries = async () => {
        const shippingCountriesResponse = await fetchShippingCountries();
        setShippingCountriesData(Object.keys(shippingCountriesResponse).map((key) => [key, shippingCountriesResponse[key]]));
    }

    const handlePaymentGateways = async () => {
        const paymentGatewaysResponse = await fetchPaymentGateways();
        setPaymentGatewaysData(paymentGatewaysResponse.data);
    }

    const couponApplyFromCart = (async (code) => {
        if (code == "") {
            alert('Please enter coupon');
        } else {
            setCouponBtnText('Wait');
            dispatch(pendingCart());
            const getcartResponse = await storeApi({ endpoint: "cart/apply-coupon", method: "POST", params: { code }, });

            if (getcartResponse.data && getcartResponse.data.status == 400) {
                dispatch(isCartStatusUpdate(true));
                setCouponBtnText('Submit');
                setCouponMessage("");
                setCouponMessageError(getcartResponse.message);
            } else {
                setCouponCode('');
                setCouponBtnText('Submit');
                dispatch(getCart(getcartResponse));
                setCouponMessageError('');
                setCouponMessage("Coupon code applied successfully.");
            }
        }
    })

    const couponRemoveFromCart = (async (code) => {
        if (code == "") {
            alert('Please enter coupon');
        } else {
            dispatch(pendingCart());
            const getcartResponse = await storeApi({ endpoint: "cart/remove-coupon", method: "POST", params: { code }, })
            dispatch(getCart(getcartResponse));
            setCouponMessage("Coupon has been removed.");
        }
    });

    const handleUserAddress = async (userId) => {
        const userAddressResponse = await AddressApi({
            endpoint: 'wp/v2/ocwma_get_address', method: "GET",
            params: {
                'user_id': userId
            },
        });
        setUserAddressData(userAddressResponse);
    }

    const updateUserAddress = async (event, userAddressDataIndex, ship_to_different_address, type) => {
        await dispatch(updateAddressStatus(ship_to_different_address));
        await dispatch(isCartStatusUpdate(false));
        var userAddressDataIndex = userAddressDataIndex[event.target.value];
        var address_data = { first_name: userAddressDataIndex.userdata.billing_first_name, last_name: userAddressDataIndex.userdata.billing_last_name, postcode: userAddressDataIndex.userdata.billing_postcode, country: userAddressDataIndex.userdata.billing_country, address_1: userAddressDataIndex.userdata.billing_address_1, address_2: userAddressDataIndex.userdata.billing_address_2, city: userAddressDataIndex.userdata.billing_city, state: userAddressDataIndex.userdata.billing_state, phone: userAddressDataIndex.userdata.billing_phone, email: userAddressDataIndex.userdata.billing_email, company: userAddressDataIndex.userdata.billing_company };

        if (type == "billing") {
            await dispatch(updateBilling(address_data));
            if (ship_to_different_address == false) {
                await dispatch(updateShipping(address_data))
            }
            setBillingAddressSelected(event.target.value);
        } else {
            await dispatch(updateShipping(address_data))
            setShippingAddressSelected(event.target.value);
        }

        await dispatch(isCartStatusUpdate(true));
    }

    useEffect(() => {
        const isSuccessful = function (paymentIntent) {
            if (paymentIntent.status == 'succeeded') {
                return true
            }
            if (paymentIntent.status == 'requires_capture') {
                return true
            }
            return false
        }
        const stripeProcess = async (result) => {
            let clientSecret = result?.data?.payment_result?.payment_details?.[2]?.value
            let verificationLink = result?.data?.payment_result?.payment_details?.[4]?.value.replace('backend/backend', 'backend')
            if (clientSecret) {
                const { paymentIntent: retrievePayment } =
                    await stripe.retrievePaymentIntent(clientSecret)
                if (retrievePayment.status == 'requires_action') {
                    const nextAction = await stripe.handleNextAction({
                        clientSecret: clientSecret,
                    })
                    if (nextAction && nextAction.paymentIntent) {
                        if (isSuccessful(nextAction.paymentIntent)) {
                            verificationLink && axios.get(verificationLink).then(function (response) {
                                //router.push("/checkout/order-received/" + result.data.order_id + "/?key=" + result.data.order_key)
                                window.location = "/checkout/order-received/" + result.data.order_id + "/?key=" + result.data.order_key;
                            })
                        } else {
                            alert('Payment failed, the PaymentIntent has a status of ' + nextAction.paymentIntent.status)
                            setCheckoutBtnText('Continue to Payment');
                        }
                    } else {
                        if (nextAction && nextAction.error && nextAction.error.message) {
                            alert(nextAction.error.message)
                            setCheckoutBtnText('Continue to Payment');
                        } else {
                            alert('failed')
                        }
                    }
                } else if (isSuccessful(retrievePayment)) {
                    verificationLink && axios.get(verificationLink).then(function (response) {
                        //router.push("/checkout/order-received/" + result.data.order_id + "/?key=" + result.data.order_key)
                        window.location = "/checkout/order-received/" + result.data.order_id + "/?key=" + result.data.order_key;
                    })
                } else {
                    alert('Unhandled PaymentIntent status ' + retrievePayment.status)
                    setCheckoutBtnText('Continue to Payment');
                }
            }
        }
        if (paymentCompleted) {
            let result = paymentCompleted
            let clientSecret =
                result?.data?.payment_result?.payment_details?.[2]?.value

            if (result.data.payment_method == 'stripe' && clientSecret) {
                stripeProcess(result)
            }
        }
    }, [paymentCompleted])
    return ((isCartReady) ?
        <div className={styles.root}>
            <div className={styles.wrapper}>
                <div className='main-content-part'>

                    <>
                        <div className='wrapper'>
                            <div className="notices-wrapper"></div>
                            <div className='checkout-content'>

                                <Formik
                                    initialValues={{
                                        billing_first_name: cart.billing_address.first_name,
                                        billing_last_name: cart.billing_address.last_name,
                                        billing_postcode: cart.billing_address.postcode,
                                        billing_country: cart.billing_address.country.toLowerCase(),
                                        billing_address_1: cart.billing_address.address_1,
                                        billing_city: cart.billing_address.city,
                                        billing_state: cart.billing_address.state,
                                        billing_phone: cart.billing_address.phone,
                                        billing_email: cart.billing_address.email,
                                        billing_company: cart.billing_address.company,
                                        ship_to_different_address: ((cart.ship_to_different_address == undefined || cart.ship_to_different_address == 'undefined' || cart.ship_to_different_address == '') ? false : cart.ship_to_different_address),
                                        shipping_first_name: cart.shipping_address.first_name,
                                        shipping_last_name: cart.shipping_address.last_name,
                                        shipping_postcode: cart.shipping_address.postcode,
                                        shipping_country: cart.shipping_address.country.toLowerCase(),
                                        shipping_address_1: cart.shipping_address.address_1,
                                        shipping_city: cart.shipping_address.city,
                                        shipping_state: cart.shipping_address.state,
                                        shipping_phone: cart.shipping_address.phone,
                                        shipping_email: cart.shipping_address.email,
                                        shipping_company: cart.shipping_address.company,
                                        payment_method: 'stripe',
                                        terms: '',
                                        subscribeNewsletter: false,
                                    }}
                                    validationSchema={CheckoutSchema}
                                    onSubmit={async (values, { resetForm, setSubmitting }) => {
                                        setSubmitting(true);
                                        var billing_address = { first_name: values.billing_first_name, last_name: values.billing_last_name, postcode: values.billing_postcode, country: values.billing_country, address_1: values.billing_address_1, address_2: values.billing_address_2, city: values.billing_city, state: values.billing_state, phone: values.billing_phone, email: values.billing_email, company: values.billing_company };
                                        var shipping_address = { first_name: values.shipping_first_name, last_name: values.shipping_last_name, postcode: values.shipping_postcode, country: values.shipping_country, address_1: values.shipping_address_1, address_2: values.shipping_address_2, city: values.shipping_city, state: values.shipping_state, phone: values.shipping_phone, email: values.shipping_email, company: values.shipping_company };
                                        let formArr = {
                                            "billing_address": billing_address,
                                            "shipping_address": values.ship_to_different_address ? shipping_address : billing_address,
                                            'create_account': 0,
                                            'customer_note': values.order_comments,
                                            'payment_method': values.payment_method,
                                            'payment_data': checkout.payment_method == 'stripe' ? stripe_payment_data.payment_data : {},
                                            'terms-field': values.terms
                                        }
                                        setCheckoutBtnText('Placing Order....');


                                        const submitData = async () => {
                                            try {
                                                const result = await axios({
                                                    url: process.env.API_URL + 'wc/store/checkout',
                                                    headers: { 'Content-Type': 'application/json' },
                                                    method: 'POST',
                                                    data: formArr,
                                                    withCredentials: true,
                                                });
                                                if (result.data.payment_result) {
                                                    setPaymentCompleted(result)
                                                    //window.location = "/checkout/order-received/" + result.data.order_id + "/?key=" + result.data.order_key;
                                                }

                                                // if (result.data.order_id && result.data.status == "processing") {

                                                //     window.location.href = "/checkout/order-received/" + result.data.order_id + "/?key=" + result.data.order_key;

                                                // }


                                            } catch (error) {
                                                alert(error.response.data.message);
                                                setCheckoutBtnText('Continue to Payment');
                                            }
                                        };

                                        if (saveStripeCardToken == "") {
                                            setSubmitting(false);
                                            setCheckoutBtnText('Continue to Payment')
                                            setErrorMessage('You need provide Debit or Credit Card detail');
                                        } else {
                                            setSubmitting(true);
                                            setErrorMessage('');
                                            submitData();
                                        }

                                    }} >
                                    {({ errors, touched, values, setFieldValue, isSubmitting }) => (



                                        <Form method="post">


                                            <FormObserver />
                                            <ConnectedFocusError />
                                            {(!isSubmitting && errorMessage != "") ?
                                                <div className="woocommerce-NoticeGroup woocommerce-NoticeGroup-checkout">
                                                    <ul className="woocommerce-error" role="alert">
                                                        <li>{errorMessage}</li>
                                                    </ul>
                                                </div>
                                                : ""
                                            }
                                            {(((Object.keys(errors).length != 0) && (Object.keys(touched).length != 0)) ?
                                                <div className="woocommerce-NoticeGroup woocommerce-NoticeGroup-checkout">
                                                    <ul className="woocommerce-error" role="alert">
                                                        {Object.keys(errors).map((key) => [key, errors[key]]).map((errorsItem, index) => {
                                                            return <li key={index} data-id={errorsItem[0]}>
                                                                <ErrorMessage component='div' name={errorsItem[0]} />
                                                            </li>
                                                        })}
                                                    </ul>
                                                </div> : '')}

                                            {((couponMessage != "") ? <div className="woocommerce-NoticeGroup woocommerce-NoticeGroup-updateOrderReview">
                                                <div className="woocommerce-message" role="alert">
                                                    {couponMessage}
                                                </div>
                                            </div> : '')}

                                            {((couponMessageError != "") ? <div className="woocommerce-NoticeGroup woocommerce-NoticeGroup-updateOrderReview">
                                                <div className="woocommerce-error" role="alert">
                                                    {couponMessageError}
                                                </div>
                                            </div> : '')}


                                            <div className="row gutters-30">
                                                <div className="col customer_details">
                                                    <div className="d-flex flex-wrap align-items-start justify-content-between heading-title">
                                                        <h2 className="entry-title">Checkout</h2>
                                                    </div>
                                                    <div className="row gutters-22">
                                                        <div className="col2-set col-md-6">
                                                            <div className="woocommerce-billing-fields">
                                                                <h3>1.  BILLING ADDRESS</h3>
                                                                <div className="woocommerce-billing-fields__field-wrapper">

                                                                    {((userIsLogin) ? <p className="form-row">
                                                                        <select className="input-text" onChange={(e) => updateUserAddress(e, userAddressData, values.ship_to_different_address, 'billing')}>
                                                                            <option value="">Billing</option>
                                                                            {userAddressData && userAddressData.map((userAddressDataItem, index) => {
                                                                                return ((userAddressDataItem.type == "billing" || userAddressDataItem.type == "common" || userAddressDataItem.type == "both") ? <option key={index} selected={billingAddressSelected == index} value={index}>{userAddressDataItem.userdata.reference_field}</option> : '')
                                                                            })}
                                                                        </select>
                                                                    </p> : '')}

                                                                    <p className="form-row form-row-first validate-required">
                                                                        <label htmlFor="billing_first_name" className="screen-reader-text">
                                                                            First Name*&nbsp;<abbr className="required" title="required">*</abbr>
                                                                        </label>
                                                                        <span className="woocommerce-input-wrapper">
                                                                            <Field className="input-text " name="billing_first_name" placeholder="First Name*" />
                                                                        </span>
                                                                    </p>


                                                                    <p className="form-row form-row-last validate-required">
                                                                        <label htmlFor="billing_last_name" className="screen-reader-text">
                                                                            Last Name*&nbsp;<abbr className="required" title="required">*</abbr>
                                                                        </label>
                                                                        <span className="woocommerce-input-wrapper">
                                                                            <Field className="input-text" name="billing_last_name" placeholder="Last Name*" />
                                                                        </span>
                                                                    </p>

                                                                    <p className="form-row address-field validate-required validate-postcode form-row-wide">
                                                                        <label htmlFor="billing_postcode" className>Postcode&nbsp;<abbr className="required" title="required">*</abbr></label>
                                                                        <span className="woocommerce-input-wrapper">
                                                                            <Field className="input-text" name="billing_postcode" placeholder="Post Code*" />
                                                                        </span>
                                                                    </p>

                                                                    <p className="form-row">
                                                                        <Field as="select" id="billing_country" className="input-text" name="billing_country" placeholder="Select a country / region…*">
                                                                            <option selected="selected" value="">Select a country / region…</option>
                                                                            {shippingCountriesData && shippingCountriesData.map((shippingCountriesItem, index) => {
                                                                                return <option key={index} value={shippingCountriesItem[0]}>{shippingCountriesItem[1]}</option>
                                                                            })}
                                                                        </Field>
                                                                    </p>

                                                                    <p className="form-row address-field validate-required form-row-wide">
                                                                        <label htmlFor="billing_address_1" className="screen-reader-text">
                                                                            &nbsp;<abbr className="required" title="required">*</abbr>
                                                                        </label>
                                                                        <span className="woocommerce-input-wrapper">
                                                                            <Field className="input-text" name="billing_address_1" placeholder="Address Line 1*" />
                                                                        </span>
                                                                    </p>

                                                                    <p className="form-row address-field form-row-wide">
                                                                        <label htmlFor="billing_address_2" className="screen-reader-text">
                                                                            &nbsp;<span className="optional">(optional)</span>
                                                                        </label>
                                                                        <span className="woocommerce-input-wrapper">
                                                                            <Field className="input-text" name="billing_address_2" placeholder="Address Line 2" />
                                                                        </span>
                                                                    </p>
                                                                    <p className="form-row address-field validate-required form-row-wide">
                                                                        <label htmlFor="billing_city" className="screen-reader-text">
                                                                            &nbsp;<abbr className="required" title="required">*</abbr>
                                                                        </label>
                                                                        <span className="woocommerce-input-wrapper">
                                                                            <Field className="input-text" name="billing_city" placeholder="City*" />
                                                                        </span>
                                                                    </p>
                                                                    <p className="form-row form-row-wide address-field">
                                                                        <span className="woocommerce-input-wrapper">
                                                                            <Field className="input-text" name="billing_state" placeholder="State/Province*" />
                                                                        </span>
                                                                    </p>
                                                                    <p className="form-row form-row form-row-wide validate-required validate-phone validate-required">
                                                                        <Field className="cus-myacc-add input-text " name="billing_phone" id="billing_phone" placeholder="Telephone Number*" />
                                                                        <label htmlFor="billing_phone" className="screen-reader-text">Telephone Number*
                                                                            <abbr className="required" title="required">*</abbr>
                                                                        </label>
                                                                    </p>
                                                                    <p className="form-row form-row form-row-wide validate-required validate-email validate-required woocommerce-invalid woocommerce-invalid-email woocommerce-invalid-phone">
                                                                        <Field className="cus-myacc-add input-text" name="billing_email" id="billing_email" placeholder="Email Address*" />
                                                                        <label htmlFor="billing_email" className="screen-reader-text">Email Address*
                                                                            <abbr className="required" title="required">*</abbr>
                                                                        </label>
                                                                    </p>
                                                                    <p className="form-row form-row-wide">
                                                                        <label htmlFor="billing_company" className="screen-reader-text">
                                                                            Company Name (Optional)&nbsp;<span className="optional">(optional)</span>
                                                                        </label>
                                                                        <span className="woocommerce-input-wrapper">
                                                                            <Field className="input-text" name="billing_company" id="billing_company" placeholder="Company Name (Optional)" />
                                                                        </span>
                                                                    </p>
                                                                </div>
                                                                <div className="woocommerce-shipping-fields">
                                                                    <h3 id="ship-to-different-address" className="step-qr">
                                                                        <span className="woocommerce-form__label woocommerce-form__label-for-checkbox checkbox">
                                                                            <Field type="checkbox" id="ship-to-different-address-checkbox" className="woocommerce-form__input woocommerce-form__input-checkbox input-checkbox" name="ship_to_different_address" /> <label htmlFor="ship_to_different_address">Ship to a different address</label>
                                                                        </span>
                                                                    </h3>

                                                                    <div className={((values.ship_to_different_address) ? "shipping_address active" : "shipping_address")}>
                                                                        {((values.ship_to_different_address) ?
                                                                            <div className="woocommerce-billing-fields__field-wrapper">

                                                                                {((userIsLogin) ? <p className="form-row">
                                                                                    <select className="input-text" onChange={(e) => updateUserAddress(e, userAddressData, values.ship_to_different_address, 'shipping')}>
                                                                                        <option value="">Shipping</option>
                                                                                        {userAddressData && userAddressData.map((userAddressDataItem, index) => {
                                                                                            return ((userAddressDataItem.type == "shipping" || userAddressDataItem.type == "common" || userAddressDataItem.type == "both") ? <option key={index} selected={((shippingAddressSelected != "") ? (shippingAddressSelected == index) : false)} value={index}>{userAddressDataItem.userdata.reference_field}</option> : '')
                                                                                        })}
                                                                                    </select> </p> : '')}

                                                                                <p className="form-row form-row-first validate-required">
                                                                                    <label htmlFor="shipping_first_name" className="screen-reader-text">
                                                                                        First Name*&nbsp;<abbr className="required" title="required">*</abbr>
                                                                                    </label>
                                                                                    <span className="woocommerce-input-wrapper">
                                                                                        <Field className="input-text " name="shipping_first_name" placeholder="First Name*" />
                                                                                    </span>
                                                                                </p>

                                                                                <p className="form-row form-row-last validate-required">
                                                                                    <label htmlFor="shipping_last_name" className="screen-reader-text">
                                                                                        Last Name*&nbsp;<abbr className="required" title="required">*</abbr>
                                                                                    </label>
                                                                                    <span className="woocommerce-input-wrapper">
                                                                                        <Field className="input-text" name="shipping_last_name" placeholder="Last Name*" />
                                                                                    </span>
                                                                                </p>

                                                                                <p className="form-row address-field validate-required validate-postcode form-row-wide">
                                                                                    <label htmlFor="shipping_postcode" className>Postcode&nbsp;<abbr className="required" title="required">*</abbr></label>
                                                                                    <span className="woocommerce-input-wrapper">
                                                                                        <Field className="input-text" name="shipping_postcode" placeholder="Post Code*" />
                                                                                    </span>
                                                                                </p>

                                                                                <p className="form-row">
                                                                                    <Field as="select" id="shipping_country" className="input-text" name="shipping_country" placeholder="Select a country / region…*">
                                                                                        <option selected="selected" value="">Select a country / region…</option>
                                                                                        {shippingCountriesData && shippingCountriesData.map((shippingCountriesItem, index) => {
                                                                                            return <option key={index} value={shippingCountriesItem[0]}>{shippingCountriesItem[1]}</option>
                                                                                        })}
                                                                                    </Field>
                                                                                </p>

                                                                                <p className="form-row address-field validate-required form-row-wide">
                                                                                    <label htmlFor="shipping_address_1" className="screen-reader-text">
                                                                                        &nbsp;<abbr className="required" title="required">*</abbr>
                                                                                    </label>
                                                                                    <span className="woocommerce-input-wrapper">
                                                                                        <Field className="input-text" name="shipping_address_1" placeholder="Address Line 1*" />
                                                                                    </span>
                                                                                </p>

                                                                                <p className="form-row address-field form-row-wide">
                                                                                    <label htmlFor="shipping_address_2" className="screen-reader-text">
                                                                                        &nbsp;<span className="optional">(optional)</span>
                                                                                    </label>
                                                                                    <span className="woocommerce-input-wrapper">
                                                                                        <Field className="input-text" name="shipping_address_2" placeholder="Address Line 2" />
                                                                                    </span>
                                                                                </p>

                                                                                <p className="form-row address-field validate-required form-row-wide">
                                                                                    <label htmlFor="shipping_city" className="screen-reader-text">
                                                                                        &nbsp;<abbr className="required" title="required">*</abbr>
                                                                                    </label>
                                                                                    <span className="woocommerce-input-wrapper">
                                                                                        <Field className="input-text" name="shipping_city" placeholder="City*" />
                                                                                    </span>
                                                                                </p>

                                                                                <p className="form-row form-row-wide address-field">
                                                                                    <span className="woocommerce-input-wrapper">
                                                                                        <Field className="input-text" name="shipping_state" placeholder="State/Province*" />
                                                                                    </span>
                                                                                </p>

                                                                                <p className="form-row form-row form-row-wide validate-required validate-phone validate-required">
                                                                                    <Field className="cus-myacc-add input-text " name="shipping_phone" id="billing_phone" placeholder="Telephone Number*" />
                                                                                    <label htmlFor="billing_phone" className="screen-reader-text">Telephone Number*
                                                                                        <abbr className="required" title="required">*</abbr>
                                                                                    </label>
                                                                                </p>

                                                                                <p className="form-row form-row form-row-wide validate-required validate-email validate-required woocommerce-invalid woocommerce-invalid-email woocommerce-invalid-phone">
                                                                                    <Field className="cus-myacc-add input-text" name="shipping_email" id="billing_email" placeholder="Email Address*" />
                                                                                    <label htmlFor="billing_email" className="screen-reader-text">Email Address*
                                                                                        <abbr className="required" title="required">*</abbr>
                                                                                    </label>
                                                                                </p>

                                                                                <p className="form-row form-row-wide">
                                                                                    <label htmlFor="shipping_company" className="screen-reader-text">
                                                                                        Company Name (Optional)&nbsp;<span className="optional">(optional)</span>
                                                                                    </label>
                                                                                    <span className="woocommerce-input-wrapper">
                                                                                        <Field className="input-text" name="shipping_company" id="shipping_company" placeholder="Company Name (Optional)" />
                                                                                    </span>
                                                                                </p>

                                                                            </div> : '')}
                                                                    </div>
                                                                </div>
                                                            </div>
                                                        </div>
                                                        <div className="col2-set col-md-6">
                                                            <div className={((isCartReady) ? "woocommerce_checkout_address delivery-address" : "woocommerce_checkout_address delivery-address processing")}  >
                                                                <h3>2.  DELIVERY OPTIONS</h3>
                                                                <ul id="shipping_method" className="woocommerce-shipping-methods">
                                                                    {isCartReady && cart && cart.shipping_rates.length > 0 && cart.shipping_rates[0].shipping_rates.map((shippingRatesItem, index) => {
                                                                        return <li key={index}>
                                                                            <Field type="hidden" name="shipping_method[0]" id={shippingRatesItem.method_id} value={shippingRatesItem.rate_id} className="shipping_method" />
                                                                            <label htmlFor={shippingRatesItem.method_id}>{shippingRatesItem.name}:
                                                                                <span className="woocommerce-Price-amount amount">
                                                                                    <bdi>
                                                                                        <span className="woocommerce-Price-currencySymbol">{shippingRatesItem.currency_symbol}{shippingRatesItem.price}</span>
                                                                                    </bdi>
                                                                                </span>
                                                                            </label>
                                                                        </li>
                                                                    })}

                                                                </ul>
                                                                <span className="wpcf7-spinner"></span>
                                                                <a onClick={() => openModal()} className="popmake-12651 pum-trigger">
                                                                    <u>Click here for delivery information</u>
                                                                </a>
                                                            </div>
                                                            <div className="woocommerce_checkout_address delivery-address">
                                                                <div id="payment" className={((isCartReady) ? "woocommerce-checkout-payment" : "woocommerce-checkout-payment processing")}>
                                                                    <h3>3.  PAYMENT METHODS</h3>
                                                                    <ul className="wc_payment_methods payment_methods methods">

                                                                        {paymentGatewaysData && paymentGatewaysData.map((paymentGatewaysItem, index) => {
                                                                            return ((paymentGatewaysItem.enabled) ?
                                                                                paymentGatewaysItem.id == 'stripe' ?
                                                                                    <li key={index} className="wc_payment_method payment_method_stripe">
                                                                                        <Field type="radio" checked={(paymentGatewaysItem.id == "stripe")} className="input-radio" onChange={(e) => { { setFieldValue(e.target.value); } }} id={paymentGatewaysItem.id} name="payment_method" value={paymentGatewaysItem.id} />
                                                                                        <label htmlFor={paymentGatewaysItem.id}>
                                                                                            Debit or Credit Card
                                                                                        </label>
                                                                                        <div className="payment_box payment_method_realex_redirect">
                                                                                            <p>{paymentGatewaysItem.description}</p>
                                                                                        </div>
                                                                                        <ElementsConsumer>
                                                                                            {({ elements, stripe }) => (
                                                                                                <div className="form">
                                                                                                    <CardForm elements={elements} stripe={stripe} />
                                                                                                </div>
                                                                                            )}
                                                                                        </ElementsConsumer>
                                                                                    </li>
                                                                                    :
                                                                                    <li key={index} className="wc_payment_method payment_method_realex_redirect">
                                                                                        <Field type="radio" checked={(paymentGatewaysItem.id == "realex_redirect")} className="input-radio" onChange={(e) => { { setFieldValue(e.target.value); } }} id={paymentGatewaysItem.id} name="payment_method" value={paymentGatewaysItem.id} />
                                                                                        <label htmlFor={paymentGatewaysItem.id}>
                                                                                            {paymentGatewaysItem.title}
                                                                                        </label>
                                                                                        <div className="payment_box payment_method_realex_redirect">
                                                                                            <p>{paymentGatewaysItem.description}</p>
                                                                                        </div>
                                                                                    </li> : '')
                                                                        })}

                                                                        {/* <li className="wc_payment_method payment_method_realex_redirect">
                                                                    <input id="payment_method_realex_redirect" type="radio" className="input-radio" name="payment_method" defaultValue="realex_redirect" defaultChecked="checked" data-order_button_text="Continue to Payment" style={{display: 'none'}} />
                                                                    <label htmlFor="payment_method_realex_redirect">
                                                                    Debit or Credit Card 
                                                                        <div className="sv-wc-payment-gateway-card-icons">
                                                                            <img src={VisaIconCheckout} alt="visa" className="sv-wc-payment-gateway-icon wc-realex-redirect-payment-gateway-icon"/><img src={MasterIconCheckout}  alt="mastercard" className="sv-wc-payment-gateway-icon wc-realex-redirect-payment-gateway-icon"/><img src={AmericanIconCheckout}  alt="amex" className="sv-wc-payment-gateway-icon wc-realex-redirect-payment-gateway-icon"/></div></label>
                                                                    <div className="payment_box payment_method_realex_redirect">
                                                                    <p>You will be directed to Global Payments to enter your payment information</p>
                                                                    </div>
                                                                </li> */}

                                                                    </ul>
                                                                    <span className="wpcf7-spinner"></span>
                                                                    <div className="form-row place-order">

                                                                        <input type="hidden" id="woocommerce-process-checkout-nonce" name="woocommerce-process-checkout-nonce" defaultValue="503baef97a" /><input type="hidden" name="_wp_http_referer" defaultValue="/?wc-ajax=update_order_review" />	</div>
                                                                </div>
                                                                <h3 className="remove-coupn">4.  DISCOUNT CODE</h3>
                                                                <p className="cus-p">Do you have a discount code?</p>
                                                                <div className="cus-coupon-sec">
                                                                    <p className="form-row form-row-first">
                                                                        <Field onChange={(e) => setCouponCode(e.target.value)} value={couponCode} type="text" name="coupon_code" className="input-text" placeholder="Enter code here" id="coupon_code" />
                                                                    </p>
                                                                    <p className="form-row form-row-last">
                                                                        <button onClick={() => couponApplyFromCart(couponCode)} type="button" className="button" id="custom_coupon" name="apply_coupon" >{couponBtnText}</button>
                                                                    </p>
                                                                </div>
                                                                <div className="clear"></div>
                                                            </div>
                                                        </div>

                                                    </div>
                                                </div>
                                                <div className="col cart-outer">
                                                    <div className={((isCartReady) ? "cart-inner" : "cart-inner processing")}>
                                                        <div id="order_review" className="woocommerce-checkout-review-order">
                                                            <div className="checkout-detail-inner">
                                                                {((Object.keys(cart).length != 0) ?
                                                                    <div className="checkout-detail-inner">
                                                                        <table className="shop_table woocommerce-checkout-review-order-table checkout-review-order">
                                                                            <thead>
                                                                                <tr>
                                                                                    <td colSpan={3}>
                                                                                        <h2>5.  REVIEW YOUR ORDER</h2>
                                                                                    </td>
                                                                                </tr>
                                                                            </thead>
                                                                            <tbody>
                                                                                <tr>
                                                                                    <td>PRODUCTS</td>
                                                                                    <td>SUBTOTAL</td>
                                                                                </tr>
                                                                                {isCartReady && cart && cart.items && cart.items.map((cartItem, index) => {
                                                                                    return <tr className="cart_item">
                                                                                        <td className="product-name pt-4 pb-4">
                                                                                            <span>{cartItem.name}<span>&nbsp;</span>
                                                                                            </span>
                                                                                        </td>
                                                                                        <td className="product-total pt-4 pb-4">
                                                                                            <span>
                                                                                                <span className="woocommerce-Price-amount amount">
                                                                                                    <bdi>
                                                                                                        <span className="woocommerce-Price-currencySymbol">{cartItem.totals.currency_symbol}{cartItem.totals.line_total}</span>
                                                                                                    </bdi>
                                                                                                </span>
                                                                                            </span>
                                                                                        </td>
                                                                                    </tr>
                                                                                })}
                                                                            </tbody>
                                                                            <tfoot>
                                                                                <tr className="cart-subtotal">
                                                                                    <th className="pt-4 pb-4">Total</th>
                                                                                    <td className="pt-4 pb-4"> </td>
                                                                                    <td className="pt-4 pb-4">
                                                                                        <span className="woocommerce-Price-amount amount">
                                                                                            <bdi>
                                                                                                <span className="woocommerce-Price-currencySymbol">{cart.totals.currency_symbol}{cart.totals.total_items}</span>
                                                                                            </bdi>
                                                                                        </span>
                                                                                    </td>
                                                                                </tr>
                                                                                {isCartReady && cart && cart.coupons && cart.coupons.map((couponsItems, index) => {
                                                                                    return <tr key={index} className="couponCode checkout-coupon-code-tr">
                                                                                        <th className="pt-0 pb-0 border-0 checkout-coupon-code-th">
                                                                                            Discount code
                                                                                        </th>
                                                                                        <td className="pt-0 pb-0 border-0"></td>
                                                                                        <td className="pt-0 pb-0 border-0 dis-remove">
                                                                                            <span className="remove-discount">
                                                                                                <a onClick={() => couponRemoveFromCart(couponsItems.code)} className="woocommerce-remove-coupon" data-coupon="ww-freeusa">X Remove</a>
                                                                                            </span>
                                                                                            -<span className="woocommerce-Price-amount amount">
                                                                                                <bdi>
                                                                                                    <span className="woocommerce-Price-currencySymbol">{couponsItems.totals.currency_symbol}{couponsItems.totals.total_discount}</span>
                                                                                                </bdi>
                                                                                            </span>
                                                                                        </td>
                                                                                    </tr>
                                                                                })}

                                                                                <tr className="cart_review_totals_shipping">
                                                                                    <th className="pt-0 pb-0 border-0">Estimated Delivery</th>
                                                                                    <td className="pt-0 pb-0 border-0"></td>
                                                                                    <td className="pt-0 pb-0 border-0">
                                                                                        <span className="shipping-amount">
                                                                                            <span className="woocommerce-Price-amount amount">
                                                                                                <bdi>
                                                                                                    <span className="woocommerce-Price-currencySymbol">{cart.totals.currency_symbol}{cart.totals.total_shipping}</span>
                                                                                                </bdi>
                                                                                            </span>
                                                                                        </span>
                                                                                    </td>
                                                                                </tr>
                                                                                <tr className="order-total">
                                                                                    <th className="pt-4 pb-4 border-0">Grand Total</th>
                                                                                    <td className="pt-4 pb-4 border-0"></td>
                                                                                    <td className="pt-4 pb-4 border-0">
                                                                                        <span className="woocommerce-Price-amount amount">
                                                                                            <bdi>
                                                                                                <span className="woocommerce-Price-currencySymbol">{cart.totals.currency_symbol}{cart.totals.total_price}</span>
                                                                                            </bdi>
                                                                                        </span>
                                                                                    </td>
                                                                                </tr>
                                                                                <tr className="additional-notes-cus">
                                                                                    <td>
                                                                                        <div className="woocommerce-additional-fields__field-wrapper">
                                                                                            <p className="form-row notes" id="order_comments_field" data-priority>
                                                                                                <label htmlFor="order_comments" className>Order notes&nbsp;
                                                                                                    <span className="optional">(optional)</span>
                                                                                                </label>
                                                                                                <span className="woocommerce-input-wrapper">
                                                                                                    <Field component="textarea" rows="5" name="order_comments" className="input-text " id="order_comments" placeholder="Comments and special instructions" />
                                                                                                </span>
                                                                                            </p>
                                                                                        </div>
                                                                                    </td>
                                                                                </tr>
                                                                                <tr className="order-total step-qr ">
                                                                                    <td colSpan={3} className="pt-4 pb-0">
                                                                                        <div className="checkbox">
                                                                                            <Field type="checkbox" name="subscribeNewsletter" id="subscribeNewsletter" />
                                                                                            <label>Subscribe to Newsletter</label>
                                                                                        </div>
                                                                                        <div className="woocommerce-terms-and-conditions-wrapper">
                                                                                            <p className="form-row validate-required">
                                                                                                <label className="woocommerce-form__label woocommerce-form__label-for-checkbox checkbox">
                                                                                                    <Field type="checkbox" className="woocommerce-form__input woocommerce-form__input-checkbox input-checkbox term-condition-tr" name="terms" id="terms" />
                                                                                                    <span className="woocommerce-terms-and-conditions-checkbox-text">I accept the <a className="popmake-12650"><u>Terms &amp; Conditions</u></a>
                                                                                                    </span>&nbsp;
                                                                                                    <span className="required">*</span>
                                                                                                </label>
                                                                                            </p>
                                                                                        </div>
                                                                                        <div className="shipping-notice" style={{ display: 'none' }}>
                                                                                            <label><h6><strong>Please note duty fee will apply</strong></h6></label>
                                                                                        </div>
                                                                                    </td>
                                                                                </tr>
                                                                                <tr className="place-order-btn">
                                                                                    <td>
                                                                                        <button type="submit" className="button alt pay-and-place-order" name="woocommerce_checkout_place_order" id="place_order" value data-value="PAY AND PLACE ORDER">{checkoutBtnText}</button>
                                                                                    </td>
                                                                                </tr>
                                                                            </tfoot>
                                                                        </table>
                                                                    </div> : '')}
                                                            </div>
                                                        </div>
                                                        <span className="wpcf7-spinner"></span>
                                                    </div>

                                                </div>
                                            </div>

                                        </Form>
                                    )}
                                </Formik>
                            </div>
                        </div>
                        <CheckoutInstructions modalIsOpen={modalIsOpen} closeModal={closeModal} />
                    </>
                </div>
            </div>
        </div> : ''
    );
};

//export default CheckoutDetails;

const WrapperCheckout = () => {
    const stripePromise = loadStripe("pk_test_51Ls6HhSB0vfnpYEJjmtlEGPUssIiNNB2k7r1AygnemeSCzm0eaf1bp2mq60sBuUGkWyZstTs1ytmWdih86is6aKF00y7mLOqpN")
    return (<Elements stripe={stripePromise}>
        <CheckoutDetails />
    </Elements>)
}
export default WrapperCheckout